/*
 * ProblemC.cc - An implementation of the problem class, it uses the 'object
 * adapter' method and uses the RandomCsp library to produce problems.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 08-Jun-1998
 *
 */

#include "ProblemC.h"



ProblemC::ProblemC
(
	ConfigurationA const & iConfiguration
): ProblemA(iConfiguration)
{
	RandomCsp = new RandomCspC();
} // ProblemC



void
ProblemC::Produce()
{
	RandomCsp->LoadFromFile(Filename);

} // Produce



bool
ProblemC::IsSolution
(
	DataA const & iData
) const
{
	throw new InternalErrorC("This function has not been implemented [ProblemC::IsSolution].");
	return false;
} // IsSolution



bool
ProblemC::Evaluate
(
	unsigned int iVariable1,
	unsigned int iVariable2,
	unsigned int iValue1,
	unsigned int iValue2
) const
{
	return RandomCsp->IsConflict(iVariable1, iVariable2, iValue1, iValue2);

} // Evaluate

// Eof ProblemC.cc
