/*
 * DataBaseProblemC.cc - An implementation of the problem class.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 05-Aug-1998
 *
 */

#include "DataBaseProblemC.h"



DataBaseProblemC::DataBaseProblemC
(
	ConfigurationA const & iConfiguration
): ProblemA(iConfiguration)
{
} // DataBaseProblemC



void
DataBaseProblemC::Produce()
{
	ifstream File;

	try
	{
		File.open(Configuration("Problem", "Filename"), ios::in);
		if (!File.is_open())
		{
			StringC ErrorString = "The datafile \"";
			ErrorString += Configuration("DataBaseProblemC", "Filename");
			ErrorString += "\" could not be opened.";
			throw new ErrorC(ErrorString);
		}

		unsigned int i = 0;
		unsigned int r = 0;
		RowC<bool> * Row = new RowC<bool>;
		Rows.push_back(Row);
		while (!File.eof())
		{
			File >> r;
			if (!File.eof())
			{
				if (i == Configuration("Problem", "SizeOfRecord"))
				{
					Row = new RowC<bool>;
					Rows.push_back(Row);
					i = 0;
				}
				Row->AddInput(r);
			}
				 
			++i;
		}

		copy(Rows.begin(), Rows.end(), ostream_iterator< RowC<bool>* >(cout, "\n"));

		File.close();
	}
	catch(ExceptionA* E)
	{
		E->Handle();
		delete E;
	}
	
} // Produce



bool
DataBaseProblemC::IsSolution
(
	DataA const & iData
) const
{
	throw new InternalErrorC("This function has not been implemented [DataBaseProblemC::IsSolution].");
	return false;
} // IsSolution



bool
DataBaseProblemC::Evaluate
(
	unsigned int iVariable1,
	unsigned int iVariable2,
	unsigned int iValue1,
	unsigned int iValue2
) const
{
	return (iValue1 == 0);

} // Evaluate



// Eof DataBaseProblemC.cc
