/*
 * DataBaseProblemC.h - An implementation of the problem class.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 04-Aug-1998
 *
 */

#ifndef DataBaseProblemC_h
#define DataBaseProblemC_h 1

#include "RowC.h"

#include <Problem/DataA.h>
#include <Problem/ProblemA.h>
#include <Error/InternalErrorC.h>
#include <ConfigurationA.h>

#include <list>

class DataA;

class DataBaseProblemC: public ProblemA
{
	public:

		DataBaseProblemC
		(
			ConfigurationA const & iConfiguration
		);

		virtual ~DataBaseProblemC()
		{
			for (list<RowC<bool> *>::iterator i = Rows.begin(); i != Rows.end(); ++i)
			{
				delete (*i);
			}

		};

		virtual void Produce();

		virtual bool IsSolution
		(
			DataA const & iData
		) const;

		//TODO: Next memberfunction isn't general and should be handled different.
		virtual bool Evaluate
		(
			unsigned int iVariable1,
			unsigned int iVariable2,
			unsigned int iValue1,
			unsigned int iValue2
		) const;

		class ConstIteratorC
		{
			public:

				ConstIteratorC
				(
					DataBaseProblemC const & iDataBaseProblem
				): DataBaseProblem(iDataBaseProblem)
				{
					Current = DataBaseProblem.Rows.begin();
				};

				virtual ~ConstIteratorC(){};

				virtual bool IsEnd()
				{
					return (Current == DataBaseProblem.Rows.end());
				};


				virtual ConstIteratorC & operator++()
				{

					++Current;
					return *this;
				};

				virtual ConstIteratorC & operator++
				(
					int iInt
				)
				{
					if (iInt == 0)
						iInt = 1;

					while (iInt > 0)
					{
						--iInt;
						++Current;
					}

					return *this;
				};


				virtual RowC<bool> & operator*() const
				{
					return (**Current);
				};

				virtual RowC<bool> * operator->() const
				{
					return (*Current);
				};

			protected:

				DataBaseProblemC const & DataBaseProblem;

				list<RowC<bool> *>::const_iterator Current;

			private:

		}; // Iterator
		
		virtual ConstIteratorC CreateConstIterator() const
		{
			 return ConstIteratorC(*this);
		};

		friend ConstIteratorC;

	protected:

	private:

		list <RowC<bool> *> Rows;

}; // DataBaseProblemC

#endif

// Eof DataBaseProblemC.h
