/*
 * ArrayC.tpl - Template class implementing an array with buildin checks.
 *
 * Part of the Evolutionary Algorithms Library.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 30-Jun-1998
 *
 */

#include "ArrayC.h"



template <class T>
void const 
ArrayC<T>::Init
(
	unsigned int const iSize
) throw(ExceptionA*)
{
	if (iSize < 1)
	{
		if (Name == 0)
			throw new InternalErrorC("Creation of array of size 0 requested.");
		else
		{
			StringC ErrorString = "Creation of array \"";
			ErrorString += *Name;
			ErrorString += "\" of size 0 requested.";
			throw new InternalErrorC(ErrorString);
		}
	}
	Size = iSize;

	Data = new T [Size];

} // Init



template <class T>
ArrayC<T>::ArrayC
(
	ArrayC<T> const & iArray
)
{
	Size = iArray.Size;
	Data = new T [Size];

	for (register unsigned int i = 0; i < Size; i++)
		Data [i] = iArray.Data [i];

} // ArrayC



template <class T>
ArrayC<T> const & 
ArrayC<T>::operator=
(
	ArrayC<T> const & iArray
)
{
	if (this != &iArray)
	{
		delete [] Data;
		Size = iArray.Size;
		Data = new T [Size];

		for (register unsigned int i = 0; i < Size; i++)
			Data [i] = iArray.Data [i];
	}
	return (*this);

} // ArrayC::operator=



template <class T>
bool const 
ArrayC<T>::operator==
(
	ArrayC<T> const & iArray
)
{
	bool Equal;

	if (this != &iArray)
	{
		if (Size != iArray.Size)
			return false;

		Equal = true;
		for (register unsigned int i = 0; i < Size && Equal; i++)
			Equal = (Data [i] == iArray.Data [i]);

		return Equal;
	}

	return true;

} // operator==



template <class T>
unsigned int ArrayC<T>::GetSize () const
{
	return Size;
	
} // GetSize


template <class T>
T & 
ArrayC<T>::operator[]
(
	unsigned int iIndex
) throw(ExceptionA*)
{
	if (iIndex >= Size)
	{
		StringC ErrorString("Out of bound index (");
		if (Name == 0)
		{
			ErrorString += StringC(iIndex);
			ErrorString += ") on array of size ";
			ErrorString += StringC(Size);
			ErrorString += ".";
		}
		else
		{
			ErrorString += StringC(iIndex);
			ErrorString += ") on array \"";
			ErrorString += *Name;
			ErrorString += "\" of size ";
			ErrorString += StringC(Size);
			ErrorString += ".";
		}
		throw new InternalErrorC(ErrorString);
	}
	return (Data [iIndex]);

} // operator[]


template <class T>
T
ArrayC<T>::operator[]
(
	unsigned int iIndex
) const throw(ExceptionA*)
{
	if (iIndex >= Size)
	{
		StringC ErrorString("Out of bound index (");
		if (Name == 0)
		{
			ErrorString += StringC(iIndex);
			ErrorString += ") on array of size ";
			ErrorString += StringC(Size);
			ErrorString += ".";
		}
		else
		{
			ErrorString += StringC(iIndex);
		    ErrorString += ") on array \"";
			ErrorString += *Name;
			ErrorString += "\" of size ";
			ErrorString += StringC(Size);
			ErrorString += ".";
		}
		throw new InternalErrorC(ErrorString);
	}
	return (Data [iIndex]);

} // operator[]


// Eof ArrayC.tpl
