/*
 * TreeA.h - Abstract class for a random-generator.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 27-Jul-1998  
 *
 */


#ifndef TreeA_h
#define TreeA_h

#include "typedef.h"

#include <vector>


/** This interface for an n-ary tree.
 *
 * @memo An n-ary tree.
 *
 * @author J.I. van Hemert
 * @version 0.1.2
 *
 */

class TreeA
{
	public:

		class TreeElementA
		{
			public:

				TreeElementA
				(
					TreeElementA & iParent
				): Parent(iParent)
				{};

				virtual ~TreeElementA();

				virtual void AddChild
				(
					TreeElementA & Child
				) =0;

				virtual vector<TreeElementA *>::iterator GetIterator() =0;
					
				virtual vector<TreeElementA *>::const_iterator GetConstIterator() =0;

			protected:
				
				vector<TreeElementA *> Children;

				TreeElementA & Parent;

			private:

		}; // TreeElementA

		class TreeConstIteratorA
		{
			public:

				TreeConstIteratorA
				(
					TreeElementA const & iTreeElement
				): TreeElement(iTreeElement)
				{
				};

				virtual ~TreeConstIteratorA() =0;

				virtual void First();
				virtual void Next();
				virtual TreeElementA const & CurrentItem() const;

			protected:

				TreeElementA const & TreeElement;

			private:

		}; // TreeConstIteratorA


		class TreeIteratorA
		{
			public:

				TreeIteratorA
				(
					TreeElementA & iTreeElement
				): TreeElement(iTreeElement)
				{
				};

				virtual ~TreeIteratorA() =0;

			protected:

				TreeElementA const & TreeElement;

			private:

		}; // TreeIteratorA

		//-----------------------------------

		TreeA()
		{
		};

		virtual ~TreeA();

	protected:

	private:

}; // TreeA

#endif
