/*
 * TreeC.tpl - Abstract class for a n-ary tree.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 02-Aug-1998  
 *
 */

#include "Common/TreeC.h"



//-------------------------------------  TreeC


//-------------------------------------  Iterator_PostorderC

template <class T>
TreeC<T>::IteratorA &
TreeC<T>::Iterator_PostorderC::operator++()
{
	if (Current == 0)
	{
		throw new InternalErrorC("Tried to walk past the end of a tree.");
	}
	
	if (Current->Right == 0)
	{
		if (StillToDo.empty())
		{
			Current = 0;
		}
		else
		{
			Current = StillToDo.top();
			StillToDo.pop();
		}
	}
	else
	{
		Current = Current->Right;
		while (Current->Child != 0)
		{
			StillToDo.push(Current);
			Current = Current->Child;
		}
	}

	return *this;
}

template <class T>
TreeC<T>::IteratorA &
TreeC<T>::Iterator_PreorderC::operator++()
{
	throw new LibraryErrorC("Not implemented yet (TreeElementC<T>::Iterator_PreorderC::operator++).");
	return *this;
}

//-------------------------------------  Iterator_PostorderC

template <class T>
void
TreeC<T>::Insert_IteratorC::AddChild
(
	T & iElement
)
{
	TreeElementC * tmp;

	if (Tree.Root == 0)
	{
		tmp = new TreeElementC(&iElement);
		Tree.Root = tmp;
	}
	else
	{
		tmp = new TreeElementC(Current, &iElement);
	}

	Current = tmp;

} // AddChild

void Swap()
{
};

// Eof TreeC.cc

