/*
 * ConfigurationC.h - A general configuration class.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 21-May-1998
 *
 */

#ifndef ConfigurationC_h
#define ConfigurationC_h 1

#include "ConfigurationA.h"

#include "Error/ExceptionA.h"
#include "Common/ArrayC.h"

#include <typeinfo>
#include <iostream.h>

class ConfigurationC: public ConfigurationA
{
	public:

		ConfigurationC
		(
			StringC const & iFileName
		) throw(ExceptionA*);

		virtual ~ConfigurationC ();

		virtual StringC const operator()
		(
			StringC const & iClass,
			StringC const & iItem
		) const;

		virtual StringC const operator()
		(
			char const * iClass,
			char const * iItem
		) const;

		virtual StringC const operator()
		(
			char const * iClass,
			StringC const & iItem
		) const;

		virtual StringC const operator()
		(
			StringC const & iClass,
			char const * iItem
		) const;

	protected:

		void const PrintToStream
		(
			ostream & iStream
		) const;

		virtual StringC const GetStringItem
		(
			StringC const & iClass,
			StringC const & iItem
		) const throw(ExceptionA*);

	private:

		unsigned int NumberOfLabels;
		unsigned int LargestString;

		typedef struct ItemS
		{
			StringC Label;
			StringC Data;
		} ItemsT;

		StringC FileName;

		//ItemsT * Items;
		ArrayC<ItemsT>* Items;


}; // ConfigurationC

#endif

// Eof ConfigurationC.h
