/*
 * PopulationC.h - simple population.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 18-Jun-1998
 *
 */

#ifndef PopulationC_h
#define PopulationC_h 1

#include "Core/PopulationA.h"

#include "Core/GenotypeC.h"
#include "Core/InitializationC.h"
#include "Core/PoolC.h"
#include "Core/PoolIteratorC.h"
#include "Problem/FitnessFunctionC.h"
#include "Reproduction/ReproductionC.h"
#include "StatisticsA.h"

#include "Problem/ProblemFactoryA.h"
#include "Reproduction/ReproductionFactoryA.h"
#include "Core/CoreFactoryA.h"


class PopulationC: public PopulationA
{
	public:

		PopulationC
		(
			ConfigurationA const & iConfiguration,
			StatisticsA & iStatistics,
		 	CoreFactoryA const & iCoreFactory,
			ProblemFactoryA const & iProblemFactory,
			ReproductionFactoryA const & iReproductionFactory,
			StopConditionA const & iStopCondition,
			ProblemA const & iProblem
		);

		virtual ~PopulationC();

		PopulationC
		(
			PopulationC const & iPopulation
		);

		
		virtual void Init();

		virtual void NextGeneration
		(
		);

		virtual GenotypeA const & GetBestIndividual() const;

	protected:

		PoolA * Pool;
		InitializationA const * Initialization;
		ReproductionA * Reproduction;
		FitnessFunctionA const * FitnessFunction;
	private:

}; // PopulationC

#endif

// Eof PopulationC.h
