/*
 * DataC.h - Abstract base class defining the interface of data in the genotype.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 26-Jun-1998
 *
 */

#ifndef DataC_h
#define DataC_h

#include "Problem/DataA.h"

#include "Common/ArrayC.h"
#include "Error/LibraryErrorC.h"
#include "typedef.h"

class DataC: public DataA
{
	public:

		class DataConstIteratorC: public DataConstIteratorA
		{
			public:
			
				DataConstIteratorC
				(
					DataA const & iData
				): DataConstIteratorA(iData)
				{
				};

				DataConstIteratorC
				(
					DataConstIteratorA const & iDataConstIterator
				);

				virtual ~DataConstIteratorC()
				{};

				virtual void First()
				{
					Cursor = 0;
				};

				virtual void Next()
				{
					++Cursor;
				};

				virtual DataType const & CurrentItem() const
				{
					return Data.Get(Cursor);
				};

				virtual bool IsDone() const
				{
					return (Cursor == Data.GetSize());
				};

				virtual DataType const & operator[]
				(
					unsigned int iIndex
				)
				{
					Cursor = iIndex;

					return Data.Get(Cursor);
				};

			protected:

			private:

		}; // DataConstIteratorC
		

		class DataIteratorC: public DataIteratorA
		{
			public:
			
				DataIteratorC
				(
					DataA & iData
				): DataIteratorA(iData)
				{
				};

				DataIteratorC
				(
					DataIteratorA const & iDataIterator
				);

				virtual ~DataIteratorC()
				{};

				virtual void First()
				{
					Cursor = 0;
				};

				virtual void Next()
				{
					++Cursor;
				};

				virtual DataType & CurrentItem() const
				{
					return Data.Get(Cursor);
				};

				virtual bool IsDone() const
				{
					return (Cursor == Data.GetSize());
				};

				virtual DataType & operator[]
				(
					unsigned int iIndex
				)
				{
					Cursor = iIndex;

					return Data.Get(Cursor);
				};

			protected:

			private:

		}; // DataIteratorC
		
		
		DataC
		(
			unsigned int iSize
		): DataA()
		{
			Array = new ArrayC<DataType>(iSize, "Data Array");
		};

		~DataC()
		{
			delete Array;
		};

		DataC
		(
			DataC const & iData
		);

		virtual DataC const & operator=
		(
			DataC const & iData
		);

		virtual DataIteratorA* CreateDataIterator()
		{
			return new DataIteratorC(*this);
		};

		virtual DataIteratorA* CreateDataIterator
		(
			DataIteratorA const & iDataIterator
		)
		{
			return new DataIteratorC(iDataIterator);
		};

		virtual DataConstIteratorA* CreateDataConstIterator() const
		{
			return new DataConstIteratorC(*this);
		};

		virtual DataConstIteratorA* CreateDataConstIterator
		(
			DataConstIteratorA const & iDataConstIterator
		) const
		{
			return new DataConstIteratorC(iDataConstIterator);
		};

		virtual DataType & Get
		(
			unsigned int iIndex
		) const
		{
			return (*Array)[iIndex];
		};

		virtual unsigned int GetSize() const
		{
			return Array->GetSize();
		};

		friend DataIteratorC;
		friend DataConstIteratorC;

		virtual void PrintToStream
		(
			ostream & in
		) const;


	protected:

		ArrayC<DataType> * Array;

	private:

}; // DataC

#endif

// Eof DataC.h
