/*
 * ReproductionFactoryC.h - Abstract base class for instantiating objects within the Fitness package.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 07-Aug-1998
 *
 */

#ifndef ReproductionFactory_C
#define ReproductionFactory_C 1

#include "Reproduction/ReproductionFactoryA.h"

#include "Reproduction/ReproductionC.h"
#include "Reproduction/SelectSurvivorsC.h"
#include "Reproduction/SelectParentsC.h"
#include "Reproduction/ChangeOperatorC.h"
#include "Reproduction/ProduceOperatorC.h"

class ProblemA;


class ReproductionFactoryC: public ReproductionFactoryA
{
	public:

		ReproductionFactoryC
		(
			ConfigurationA const & iConfiguration,
			RandomA const & iRandom
		): ReproductionFactoryA(iConfiguration, iRandom)
		{
		};

		virtual ~ReproductionFactoryC()
		{};

		virtual ReproductionA* CreateReproduction
		(
			CoreFactoryA const & iCoreFactory,
			ProblemFactoryA const & iProblemFactory,
			ProblemA const & iProblem,
			StopConditionA const & iStopCondition
		) const;

		virtual SelectParentsA* CreateSelectParents() const;
		virtual SelectSurvivorsA* CreateSelectSurvivors() const;

		virtual ChangeOperatorA* CreateChangeOperator
		(
			ProblemFactoryA const & iProblemFactory,
			ProblemA const & iProblem
		) const;
			
		virtual ProduceOperatorA* CreateProduceOperator
		(
			CoreFactoryA const & iCoreFactory,
			ProblemFactoryA const & iProblemFactory,
			ProblemA const & iProblem
		) const;

	protected:

	private:

}; // ReproductionFactoryC

#endif

// Eof ReproductionFactoryC.h
