/*
 * StatisticsC.h - A basic statistics class.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 17-Jun-1998
 *
 */

#ifndef StatisticsC_h
#define StatisticsC_h 1

#include "StatisticsA.h"

#include "typedef.h"

class StatisticsC: public StatisticsA
{
	public:

		StatisticsC
		(
			ConfigurationA const & iConfiguration
		);

		virtual ~StatisticsC();

		virtual void ResetRunStatistics();

		virtual void CollectBasicRunStatistics
		(
			bool iIsSucces,
			FitnessType const & iBestFitness
		);

		virtual void RegisterEvaluation()
		{
			++NumberOfEvaluations;
		};

		virtual void RegisterEvaluations
		(
			unsigned long int iEvaluations
		)
		{
			NumberOfEvaluations += iEvaluations;
		};

		virtual unsigned long int GetNumberOfEvaluations() const
		{
			return NumberOfEvaluations;
		};

	protected:

		virtual void PrintToStream
		(
			ostream & iOstream
		) const;

	private:

}; // StatisticsC

#endif

// Eof StatisticsC.h
