/*
 * EvolutionaryAlgorithmC.cc - Simple evolutionary algorithm.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 10-Jun-1998
 *
 */

#include "Core/EvolutionaryAlgorithmC.h"



EvolutionaryAlgorithmC::EvolutionaryAlgorithmC
(
	ConfigurationA const & iConfiguration,
	StatisticsA & iStatistics,
	CoreFactoryA const & iCoreFactory,
	ProblemFactoryA const & iProblemFactory,
	ReproductionFactoryA const & iReproductionFactory,
	ProblemA const & iProblem
): EvolutionaryAlgorithmA(iConfiguration, iStatistics, iCoreFactory, iProblemFactory, iReproductionFactory, iProblem)
{
	StopCondition = CoreFactory.CreateStopCondition();

	Population = CoreFactory.CreatePopulation(Statistics, ProblemFactory, ReproductionFactory, *StopCondition, Problem);

} // EvolutionaryAlgorithmC



EvolutionaryAlgorithmC::~EvolutionaryAlgorithmC()
{
	delete Population;
	delete StopCondition;

} // ~EvolutionaryAlgorithmC



void
EvolutionaryAlgorithmC::Run()
{
	Population->Init();

	while (!StopCondition->IsStopRequested())
	{
		Population->NextGeneration();
	}

	Statistics.CollectBasicRunStatistics(StopCondition->IsSolutionFound(), Population->GetBestIndividual().GetFitness());

} // Run



// Eof EvolutionaryAlgorithmC.cc
