/*
 * GenotypeC.cc - Simple implementation of a genotype.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 10-Jun-1998
 *
 */

#include "Core/GenotypeC.h"

GenotypeC::GenotypeC
(
	ConfigurationA const & iConfiguration,
	ProblemFactoryA const & iProblemFactory
): GenotypeA(iConfiguration, iProblemFactory)
{
	//REM: FitnessValue = ProblemFactory.CreateFitnessValue();

	FitnessValue = 0;
	Data = ProblemFactory.CreateData(iConfiguration("Genotype", "Size"));

} // GenotypeC



GenotypeC::~GenotypeC()
{
	//REM: delete FitnessValue;
	delete Data;

} // ~GenotypeC



GenotypeC::GenotypeC
(
	ConfigurationA const & iConfiguration,
 	ProblemFactoryA const & iProblemFactory,
	GenotypeA const & iGenotype
): GenotypeA(iConfiguration, iProblemFactory)
{
	//REM: FitnessValue = ProblemFactory.CreateFitnessValue();
	Data = ProblemFactory.CreateData(iConfiguration("Genotype", "Size"));

	Copy(iGenotype);

} // GenotypeC



void
GenotypeC::Init
(
	InitializationA const & iInitialization
)
{
	iInitialization(*Data);

} // Init



GenotypeC const & 
GenotypeC::operator=
(
	GenotypeA const & iGenotype
)
{
	Copy(iGenotype);

	return *this ;

} // operator=



void
GenotypeC::Evaluate
(
	FitnessFunctionA const & iFitnessFunction
)
{
	iFitnessFunction.Calculate(FitnessValue, *Data);

} // Evaluate



void
GenotypeC::Copy
(
	GenotypeA const & iGenotype
)
{
	*Data = iGenotype.GetData();

	FitnessValue = iGenotype.GetFitness();

} // Copy


/*
DataA const & 
GenotypeC::GetData() const
{
	 return *Data;
	 
} // GetData



void
GenotypeC::SetData
(
	DataA & iData
)
{
	Data = &iData;

} // SetData
*/


/*
void  
GenotypeC::SetFitnessValue
(
	FitnessValueA & iFitnessValue
)
{
	FitnessValue = &iFitnessValue;

} // SetFitnessValue
*/

/*
bool 
GenotypeC::IsEqual
(
	GenotypeA const & iGenotype
) const
{
		return (FitnessValue == iGenotype.FitnessValue);

} // IsEqual



bool 
GenotypeC::IsSmaller
(
	GenotypeA const & iGenotype
) const
{
	return (FitnessValue > iGenotype.FitnessValue);

} // IsSmaller


bool 
GenotypeC::IsGreater
(
	GenotypeA const & iGenotype
) const
{
	return (FitnessValue < iGenotype.FitnessValue);

} // IsGreater
*/




void GenotypeC::PrintToStream
(
	ostream & in
) const
{
	in << GetUniqueID() << " : " << *Data << " : " << FitnessValue;

} // PrintToStream



// Eof GenotypeC.cc
