/*
 * PoolIteratorC.cc - Simple implementation of a population pool.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 07-Jun-1998
 *
 */

#include "Core/PoolIteratorC.h"



PoolIteratorC::PoolIteratorC
(
	PoolA & iPool
)
{
	Cursor = 0;

	Pool = &iPool;

} // PoolIteratorC


void PoolIteratorC::First()
{
	Cursor = &(Pool->GetThePool());
};

void PoolIteratorC::Next()
{
	if (Cursor != 0)
		Cursor = &(Cursor->GetNextPoolElement());
	else
		throw new InternalErrorC("Tried to walk beyond the last individual in the pool.");
};

void PoolIteratorC::Delete()
{
	if (Cursor != 0)
	{
		PoolElementA * Tmp = Cursor;
		Next();
		Pool->Delete(*Tmp);
	}
	else
		throw new InternalErrorC("Tried to free a nonexisting item in the pool using a reverseiterator");
};

GenotypeA & PoolIteratorC::CurrentItem() const
{
	if (Cursor != 0)
		return Cursor->GetGenotype();
	else
		throw new InternalErrorC("Requested an item from the pool with an iterator which is not pointing to something.");
};

bool PoolIteratorC::IsDone() const
{
	return (Cursor == 0);
};


//TODO: Finish copy ctors for PoolIterator.
/*PoolIteratorC::PoolIteratorC
(
	PoolIteratorC const & iPoolIterator
)
{

} // PoolIteratorC


PoolIteratorC const &
PoolIteratorC::PoolIteratorC
(
	PoolIteratorC const & iPoolIterator
)
{

} // PoolIteratorC
*/



PoolIteratorC::~PoolIteratorC()
{
} // ~PoolIteratorC


//-------------------------------------------------------


PoolConstIteratorC::PoolConstIteratorC
(
	PoolA const & iPool
)
{
	Cursor = 0;

	Pool = &iPool;

} // PoolConstIteratorC


void PoolConstIteratorC::First()
{
	Cursor = &(Pool->GetThePool());
};

void PoolConstIteratorC::Next()
{
	if (Cursor != 0)
		Cursor = &(Cursor->GetNextPoolElement());
	else
		throw new InternalErrorC("Tried to walk beyond the last individual in the pool.");
};

GenotypeA const &
PoolConstIteratorC::CurrentItem() const
{
	if (Cursor != 0)
		return Cursor->GetGenotype();
	else
		throw new InternalErrorC("Requested an item from the pool with an iterator which is not pointing to something.");
};

bool 
PoolConstIteratorC::IsDone() const
{
	return (Cursor == 0);
};


//TODO: Finish copy ctors for PoolConstIterator.
/*PoolConstIteratorC::PoolConstIteratorC
(
	PoolConstIteratorC const & iPoolConstIterator
)
{

} // PoolConstIteratorC


PoolConstIteratorC const &
PoolConstIteratorC::PoolConstIteratorC
(
	PoolConstIteratorC const & iPoolConstIterator
)
{

} // PoolConstIteratorC
*/



PoolConstIteratorC::~PoolConstIteratorC()
{
} // ~PoolConstIteratorC



// Eof PoolIteratorC.cc
