/*
 * PopulationC.cc - Simple implementation of a population.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 11-Jun-1998
 *
 */

#include "Core/PopulationC.h"



PopulationC::PopulationC
(
	ConfigurationA const & iConfiguration,
	StatisticsA & iStatistics,
	CoreFactoryA const & iCoreFactory,
	ProblemFactoryA const & iProblemFactory,
	ReproductionFactoryA const & iReproductionFactory,
	StopConditionA const & iStopCondition,
	ProblemA const & iProblem
): PopulationA(iConfiguration, iStatistics, iCoreFactory, iProblemFactory, iReproductionFactory, iStopCondition, iProblem)
{
	Initialization = CoreFactory.CreateInitialization(Problem);

	Reproduction = ReproductionFactory.CreateReproduction(CoreFactory, ProblemFactory, Problem, StopCondition);

	FitnessFunction = ProblemFactory.CreateFitnessFunction(StopCondition, Statistics, Problem);

	Pool = CoreFactory.CreatePool();

	try
	{
		Size = Configuration("Population", "Size");
	}
	catch(ExceptionA* E)
	{
		E->Handle();
		delete E;
		Size = 0;
	}

} // PopulationC



PopulationC::PopulationC
(
	PopulationC const & iPopulation
): PopulationA(iPopulation.Configuration, iPopulation.Statistics, iPopulation.CoreFactory, iPopulation.ProblemFactory, iPopulation.ReproductionFactory, iPopulation.StopCondition, iPopulation.Problem)
{
	Size = iPopulation.Size;

	//TODO: Check if pool has a working operator=
	Pool = iPopulation.Pool;

} // PopulationC



PopulationC::~PopulationC()
{
#ifdef DEBUG
	cerr << *Pool;
#endif DEBUG

	delete Initialization;
	delete Pool;
	delete FitnessFunction;
	delete Reproduction;

} // ~PopulationC



void
PopulationC::Init()
{
	/*
	PoolIteratorA * PoolIterator = Pool->CreatePoolIterator();

	PoolIterator->First();
	while (!PoolIterator->IsDone())
	{
		PoolIterator->CurrentItem().Init(*Initialization);
		PoolIterator->CurrentItem().Evaluate(*FitnessFunction);
		PoolIterator->Next();
	}

	delete PoolIterator;

#ifdef DEBUG
	cerr << *Pool;
#endif DEBUG
*/
	unsigned int i = 0;
	while (i < Size)
	{
		//TODO: Genotypes should only be produced in constructor
		GenotypeA * Genotype = CoreFactory.CreateGenotype(ProblemFactory);
		Genotype->Init(*Initialization);
		Genotype->Evaluate(*FitnessFunction);

		Pool->Add(*Genotype);

		++i;
	}

#ifdef DEBUG
	cerr << *Pool;
#endif DEBUG
} // Init



void
PopulationC::NextGeneration()
{
	(*Reproduction)(*Pool, *FitnessFunction, Statistics);

	//REM: cout << GetBestIndividual() << endl;

} // NextGeneration



GenotypeA const &
PopulationC::GetBestIndividual() const
{
	return Pool->GetThePool().GetGenotype();
};



// Eof PopulationC.cc
