/*
 * FitnessFunctionC.cc - Simple implementation of a fitnessfunction.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 18-Jun-1998
 *
 */

#include "Problem/FitnessFunctionC.h"



FitnessFunctionC::FitnessFunctionC
(
	ConfigurationA const & iConfiguration,
	StatisticsA & iStatistics,
	StopConditionA const & iStopCondition,
	ProblemA const & iProblem
): FitnessFunctionA(iConfiguration, iStatistics, iStopCondition, iProblem)
{
} // FitnessFunctionC


FitnessFunctionC::~FitnessFunctionC()
{
} // ~FitnessFunctionC



void const
FitnessFunctionC::Calculate
(
	FitnessType & iFitnessValue,
	DataA const & iData
) const
{
	DataA::DataConstIteratorA * DataIterator1 = iData.CreateDataConstIterator();

	iFitnessValue = 0;
	unsigned int i = 0;
	DataIterator1->First();
	while (!DataIterator1->IsDone())
	{
		unsigned int j = i;
		DataA::DataConstIteratorA * DataIterator2 = iData.CreateDataConstIterator(*DataIterator1);
		while (!DataIterator2->IsDone())
		{
			if (Problem.Evaluate(i, j, DataIterator1->CurrentItem(), DataIterator2->CurrentItem()))
				++iFitnessValue;
			DataIterator2->Next();
			++j;
		}
		delete DataIterator2;
		DataIterator1->Next();
		++i;
	}
	
	delete DataIterator1;

	if (iFitnessValue == 0)
		StopCondition.FoundSolution();

	Statistics.RegisterEvaluation();

} // Calculate




// Eof FitnessFunctionC.cc
