/*
* testTreeC.cc - Tests the TreeC class.
*
* Part of the Library for Evolutionary Algorithm Programming.
*
* Copyright (C) 1998 J.I. van Hemert
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Author: J.I. van Hemert
* Last Revision: 01-Aug-1998  
*
*/

#include "testTreeC.h"
#include <Common/TreeC.tpl>

class EmptyC
{
	public:
		EmptyC(int _i = 24): i(_i) {};

		int test() const
		{
			return i;
		};
		
		EmptyC const & operator=
		(
			EmptyC const & iEmpty
		)
		{
			i = iEmpty.i;

			return *this;
		};

	protected:
		int i;

} Empty;

void TreeC4()
{
	try
	{
		TreeC<EmptyC> t;
		TreeC<EmptyC> t2;

		TreeC<EmptyC>::Insert_IteratorC ii = t.CreateInsert_IteratorC(t.CreateIterator_Postorder());
		EmptyC * a = new EmptyC(1);
		ii.AddChild(*a);
		a = new EmptyC(2);
		ii.AddChild(*a);
		a = new EmptyC(3);
		ii.AddChild(*a);

		TreeC<EmptyC>::Insert_IteratorC ii2 = t.CreateInsert_IteratorC(t.CreateIterator_Postorder());
		a = new EmptyC(4);
		ii2.AddChild(*a);
		a = new EmptyC(5);
		ii2.AddChild(*a);
		a = new EmptyC(6);
		ii2.AddChild(*a);

		Swap();

		TreeC<EmptyC>::Insert_IteratorC ii3 = t.CreateInsert_IteratorC(++t.CreateIterator_Postorder());

		// boompje inplanten

		TreeC<EmptyC>::Iterator_PostorderC i = t.CreateIterator_Postorder();
		while (!i.IsEnd())
		{
			cout << "  " << i->test();
			++i;
		}
	}
	catch (ExceptionA* e)
	{
		 e->Handle;
		 delete e;
	}

}

// --------------------------------

void TreeC3()
{
	// Testing handling of end of tree iterator.
	cerr << "iterator Iterator_PostorderC::IsEnd: ";

	TreeC<EmptyC> t;
	TreeC<EmptyC>::Insert_IteratorC ii = t.CreateInsert_IteratorC(t.CreateIterator_Postorder());
	EmptyC b;
	ii.AddChild (b);

	TreeC<EmptyC>::Iterator_PostorderC i = t.CreateIterator_Postorder();
	bool failed = false;
	try
	{
		if ((*i).test() != 24)
			failed = true;
		++i;
		if (!i.IsEnd())
			failed = true;
		++i;
		// An exception should be thrown by now,
		// the next line should _not_ be executed
		failed = true;

	}
	catch (ExceptionA* e)
	{
		delete e;
	}

	if (failed)
		cerr << "failed\n";
	else
		cerr << "ok\n";
}

//----------------------------------

void TreeC2()
{
	cerr << "Insert_IteratorC::operator+: ";

	TreeC<EmptyC> t;
	TreeC<EmptyC>::Insert_IteratorC ii = t.CreateInsert_IteratorC(t.CreateIterator_Postorder());
	EmptyC b;
	ii.AddChild (b);
	EmptyC c(100);
	ii.AddChild (c);

	TreeC<EmptyC>::Iterator_PostorderC i = t.CreateIterator_Postorder();
	bool failed = false;
	try
	{
		if ((*i).test() != 100)
			failed = true;

		++i;
		if ((*i).test() != 24)
			failed = true;


	}
	catch (ExceptionA* e)
	{
		failed = true;
		cerr << "(exception) ";
		delete e;
	}

	if (failed)
		cerr << "failed\n";
	else
		cerr << "ok\n";
}

//----------------------------------

void TreeC1()
{
	cerr << "Iterator_PostorderC::AddChild: ";

	TreeC<EmptyC> t;
	TreeC<EmptyC>::Insert_IteratorC ii = t.CreateInsert_IteratorC(t.CreateIterator_Postorder());
	EmptyC b;
	ii.AddChild (b);

	TreeC<EmptyC>::Iterator_PostorderC i = t.CreateIterator_Postorder();
	try
	{
		if ((*i).test() != 24)
			cerr << "failed\n";
		else
			cerr << "ok\n";
		
	}
	catch (ExceptionA* e)
	{
		cerr << "(exception) failed\n";
		delete e;
	}
}

//-----------------------------------------

void TreeC0()
{
	cerr << "TreeC::constructor...";

	bool failed = false;
	try
	{
		TreeC<EmptyC> * a = new TreeC<EmptyC>();

		cerr << "destructor: ";

		delete a;
	}
	catch (ExceptionA* e)
	{
		cerr << "(exception) ";
		failed = true;
		delete e;
	}

	if (failed)
		cerr << "failed";
	else
		cerr << "ok\n";
}

int testTreeC()
{
	cerr << "<testing TreeC>\n";

	TreeC0();
	TreeC1();
	TreeC2();
	TreeC3();
	TreeC4();

	return 0;
}
